@@ -1,13 +1,14 @@
 package Net::Stomp::Producer;
 {
-  $Net::Stomp::Producer::VERSION = '1.001';
+  $Net::Stomp::Producer::VERSION = '1.1';
 }
 {
   $Net::Stomp::Producer::DIST = 'Net-Stomp-Producer';
 }
 use Moose;
 use namespace::autoclean;
-with 'Net::Stomp::MooseHelpers::CanConnect';
+with 'Net::Stomp::MooseHelpers::CanConnect' => { -version => '1.1' };
+with 'Net::Stomp::MooseHelpers::ReconnectOnFailure';
 use MooseX::Types::Moose qw(CodeRef HashRef);
 use Net::Stomp::Producer::Exceptions;
 use Class::Load 'load_class';
@@ -16,35 +17,6 @@ use Try::Tiny;
 # ABSTRACT: helper object to send messages via Net::Stomp
 
 
-# we automatically send the C<connect> frame
-around _build_connection => sub {
-    my ($orig,$self,@etc) = @_;
-    my $conn = $self->$orig(@etc);
-    $self->_connect($conn);
-    return $conn;
-};
-
-sub _connect {
-    my ($self,$connection) = @_;
-
-    try {
-        my $server = $self->current_server;
-        my %headers = (
-            %{$self->connect_headers},
-            %{$server->{connect_headers} || {}},
-        );
-        $connection->connect(\%headers);
-    } catch {
-        Net::Stomp::MooseHelpers::Exceptions::Stomp->throw({
-            stomp_error => $_
-        });
-    };
-}
-
-
-sub connect { warn "No-op, Net::Stomp::Producer connects on its own" }
-
-
 has serializer => (
     isa => CodeRef,
     is => 'rw',
@@ -88,7 +60,7 @@ sub send {
     my %actual_headers=(
         %{$self->default_headers},
         %$headers,
-        'content-length' => length($body),
+        #'content-length' => length($body),
         body => $body,
     );
 
@@ -99,7 +71,9 @@ sub send {
             unless m{^/};
     }
 
-    $self->connection->send(\%actual_headers);
+    $self->reconnect_on_failure(
+        sub{ $_[0]->connection->send($_[1]) },
+        \%actual_headers);
 
     return;
 }
@@ -176,7 +150,7 @@ Net::Stomp::Producer - helper object to send messages via Net::Stomp
 
 =head1 VERSION
 
-version 1.001
+version 1.1
 
 =head1 SYNOPSIS
 
@@ -315,12 +289,6 @@ L</make_transformer> instantiates a transformer class.
 
 =head1 METHODS
 
-=head2 C<connect>
-
-Since the connection is set up automatically, this method (usually
-provided by L<Net::Stomp::MooseHelpers::CanConnect>) is overridden to
-be a no-op and warn. Don't call it.
-
 =head2 C<send>
 
   $p->send($destination,\%headers,$body);
